/*
 * CameraCapture.cpp
 *
 *  Created on: 9 dec. 2011
 *      Author: henrique
 */

#include "CameraCapture.h"

#include <iostream>
#include <stdio.h>
#include <errno.h>

CameraCapture::CameraCapture() {

	/*
	 * Setting capture device
	 */
	this->capture = cvCaptureFromCAM(-1);

	/*
	 * Retrieving frame to set image size
	 */
	this->lastImage = cvQueryFrame(capture);
	this->imageWidth = lastImage->width;

}

IplImage * CameraCapture::getImage() {

	/*
	 * Retrieving camera frame
	 */
	 this->lastImage = cvQueryFrame(capture);

	 /*
	  * New image size
	  */
	 CvSize imageSize;
	 imageSize.height = this->lastImage->width;
	 imageSize.width = this->lastImage->height;

	 /*
	  * Creating the new image which will hold the rotated camera capture
	  */
	 IplImage *dst = cvCreateImage(imageSize, this->lastImage->depth, this->lastImage->nChannels);

	 /*
	  * Rotation matrix parameters
	  */
	 float lRotateValues[6];
	 lRotateValues[0] = 0.0;
	 lRotateValues[1] = 1.0;
	 lRotateValues[3] = -lRotateValues[1];
	 lRotateValues[4] = lRotateValues[0];
	 lRotateValues[2] = this->lastImage->width * 0.5f;
	 lRotateValues[5] = this->lastImage->height * 0.5f;

	 /*
	  * Creating rotation matrix
	  */
	 CvMat lRotateMatrix = cvMat(2, 3, CV_32F, lRotateValues);

	 /*
	  * Rotating original image
	  */
	 cvGetQuadrangleSubPix(this->lastImage, dst, &lRotateMatrix);

	 /*
	  * Keeping the last captured image
	  */
	 this->lastImage = cvCloneImage(dst);

	 /*
	  * Deallocating auxiliary image
	  */
	 cvReleaseImage(&dst);

	 /*
	  * Handle key events - it is necessary to the successfully image processing
	  */
	 cvWaitKey(10);

	 return this->lastImage;
}

CameraCapture::~CameraCapture() {

	/*
	 * Releasing camera device
	 */
	cvReleaseCapture(&this->capture);

	/*
	 * Handle key events
	 */
	cvWaitKey(0);
}

