/*
 * ImageProcessing.cpp
 *
 *  Created on: Jan 6, 2012
 *      Author: henriquenogueira
 */

#include "ImageProcessing.h"
#include "CameraCapture.h"

#include <highgui.h>
#include <math.h>



extern int msgQueueImageToIa;
extern double alpha0;
extern double alpha1;
extern bool inAttackMode;

/*
 * Corner control global variables
 */
bool waitingCorners = true;
unsigned short int cornerCount = 0;
Corner cornersEventHandler[CORNERS];


/*
 * Colors control global variables
 */
bool waitingColors = true;
unsigned short int colorCount = 0;
int hEventHandler[COLORS], sEventHandler[COLORS], vEventHandler[COLORS];
int toleranceH[3] = {4, 27, 10}, toleranceS[3]= {30, 77, 50}, toleranceV[3] = {30, 91, 50};

/*
 * Direction controller
 */
int directionX[] = {1, 1, 0, -1, -1, -1, 0, 1};
int directionY[] = {0, 1, 1, 1, 0, -1, -1, -1};

/*
 * Image control variable
 */
IplImage *image;

void on_trackbar(int receivedValue, void * userData) {
	int value = receivedValue;
	int * parameter = (int *) userData;

	*parameter = value;
}

void onMouseClickField(int event, int x, int y, void * params) {
	//	if (event == CV_EVENT_LBUTTONDOWN && waitingCorners) {
	//
	//		printf("Corner detected (%d, %d)\n", x, y);
	//
	//		cornersEventHandler[cornerCount].x = x;
	//		cornersEventHandler[cornerCount].y = y;
	//		cornerCount++;
	//		waitingCorners = cornerCount < 4;
	//	}

	cornersEventHandler[0].x = 134;
	cornersEventHandler[0].y = 101;
	cornersEventHandler[1].x = 392;
	cornersEventHandler[1].y = 121;
	cornersEventHandler[2].x = 23;
	cornersEventHandler[2].y = 549;
	cornersEventHandler[3].x = 435;
	cornersEventHandler[3].y = 575;

	waitingCorners = false;

}

void onMouseClickFieldPrint(int event, int x, int y, void * params) {
	if (event == CV_EVENT_LBUTTONDOWN) {
		printf("Point detected (%d, %d)\n", x, y);
	}
}


void onMouseClickColor(int event, int x, int y, void * params) {
	if (event == CV_EVENT_LBUTTONDOWN and waitingColors) {
		IplImage * hsv;
		hsv = cvCloneImage(image);
		cvCvtColor(hsv, hsv, CV_BGR2HSV);

		CvScalar pixel;
		pixel = cvGet2D(hsv, y, x);

		hEventHandler[colorCount] = (int) pixel.val[0];
		sEventHandler[colorCount] = (int) pixel.val[1];
		vEventHandler[colorCount] = (int) pixel.val[2];

		printf("Color detected HSV(%d, %d, %d)\n", hEventHandler[colorCount], sEventHandler[colorCount], vEventHandler[colorCount]);

		cvReleaseImage(&hsv);

		colorCount++;
		waitingColors = colorCount < 3;
	}
}


ImageProcessing::ImageProcessing() {
}

void ImageProcessing::initField() {

	cvNamedWindow("Field configuration", 1);
	cvSetMouseCallback("Field configuration", (CvMouseCallback) onMouseClickField);
	CameraCapture *camera = new CameraCapture();

	image = NULL;
	while (waitingCorners) {
		image = camera->getImage();

		if (!image)
			continue;

		cvShowImage("Field configuration", image);
		cvWaitKey(10);
	}
	cvDestroyWindow("Field configuration");

	for(int i = 0; i < CORNERS; i++)
		this->corners[i] = cornersEventHandler[i];

	delete camera;
}

void ImageProcessing::initColors() {

	cvNamedWindow("Colors configuration", 1);
	cvSetMouseCallback("Colors configuration", (CvMouseCallback) onMouseClickColor);
	CameraCapture *camera = new CameraCapture();

	image = NULL;
	while (waitingColors) {
		image = camera->getImage();

		if (!image)
			continue;

		cvShowImage("Colors configuration", image);
		cvWaitKey(10);
	}
	cvDestroyWindow("Colors configuration");
	delete camera;
}

void ImageProcessing::init() {
	this->initField();
	this->initColors();
}

void ImageProcessing::start() {
	pthread_create(&thread, NULL, run, NULL);
}


void* ImageProcessing::run(void* data) {

	CvMat* mmat = cvCreateMat(3,3,CV_32FC1);
	CameraCapture *camera = new CameraCapture();
	Position ballPositionOld, robotRightOld, robotLeftOld;
	struct msgImageToIa msgToSend;
	unsigned int prio = 0;
	IplImage *src, *dst;

	/*
	 * Getting original image
	 */
	src = camera->getImage();

	/*
	 * Perspective matrix configuration
	 */
	CvPoint2D32f srcQuad[4], dstQuad[4];
	dstQuad[0].x = 0;
	dstQuad[0].y = 0;

	dstQuad[1].x = src->width - 1;
	dstQuad[1].y = 0;

	dstQuad[2].x = 0;
	dstQuad[2].y = src->height - 1;

	dstQuad[3].x = src->width - 1;
	dstQuad[3].y = src->height - 1;

	srcQuad[0].x = cornersEventHandler[0].x;
	srcQuad[0].y = cornersEventHandler[0].y;

	srcQuad[1].x = cornersEventHandler[1].x;
	srcQuad[1].y = cornersEventHandler[1].y;

	srcQuad[2].x = cornersEventHandler[2].x;
	srcQuad[2].y = cornersEventHandler[2].y;

	srcQuad[3].x = cornersEventHandler[3].x;
	srcQuad[3].y = cornersEventHandler[3].y;

	/*
	 * Getting transformed matrix
	 */
	cvGetPerspectiveTransform(
			srcQuad,
			dstQuad,
			mmat
	);

	cvNamedWindow("PERSPECTIVE", 1);
	//cvNamedWindow("WHITE MASK", 1);
	cvNamedWindow("BLUE MASK", 1);
	cvNamedWindow("BALL MASK", 1);

	cvSetMouseCallback("PERSPECTIVE", (CvMouseCallback) onMouseClickFieldPrint);

	//cvCreateTrackbar2("H Tolerance", "WHITE MASK", &toleranceH[0], MAX_TOLERANCE, on_trackbar, &toleranceH[0]);
	cvCreateTrackbar2("H Tolerance", "BLUE MASK", &toleranceH[1], MAX_TOLERANCE, on_trackbar, &toleranceH[1]);
	cvCreateTrackbar2("H Tolerance", "BALL MASK", &toleranceH[2], MAX_TOLERANCE, on_trackbar, &toleranceH[2]);

	//cvCreateTrackbar2("S Tolerance", "WHITE MASK", &toleranceS[0], MAX_TOLERANCE, on_trackbar, &toleranceS[0]);
	cvCreateTrackbar2("S Tolerance", "BLUE MASK", &toleranceS[1], MAX_TOLERANCE, on_trackbar, &toleranceS[1]);
	cvCreateTrackbar2("S Tolerance", "BALL MASK", &toleranceS[2], MAX_TOLERANCE, on_trackbar, &toleranceS[2]);

	//cvCreateTrackbar2("V Tolerance", "WHITE MASK", &toleranceV[0], MAX_TOLERANCE, on_trackbar, &toleranceV[0]);
	cvCreateTrackbar2("V Tolerance", "BLUE MASK", &toleranceV[1], MAX_TOLERANCE, on_trackbar, &toleranceV[1]);
	cvCreateTrackbar2("V Tolerance", "BALL MASK", &toleranceV[2], MAX_TOLERANCE, on_trackbar, &toleranceV[2]);


	CvMemStorage * storage = cvCreateMemStorage(0);

	CvFont font;
	double hScale;
	double vScale;
	int    lineWidth;

	char cordRobot1[20], cordRobot2[20];
	char alpha0String[20], alpha1String[20], inAttackModeString[20];

	Position robotZeroBluePosition, robotOneBluePosition, robotZeroWhitePosition, robotOneWhitePosition;
	Position ballPosition, robotZeroPosition, robotOnePosition;

	IplImage * imageHSV;
	IplImage * ballColor;
	//IplImage * whiteRobotColor;
	IplImage * blueRobotColor;

	ballPositionOld.x = -1;
	ballPositionOld.y = -1;
	robotLeftOld.x = -1;
	robotLeftOld.y = -1;
	robotRightOld.x = -1;
	robotRightOld.y = -1;

	while(true) {
		src = camera->getImage();
		dst = cvCloneImage(src);
		dst->origin = src->origin;

		/*
		 * Wrapping perspective
		 */
		cvWarpPerspective(src, dst, mmat);

		cvSmooth(dst, dst, CV_GAUSSIAN, 3);

		imageHSV = cvCreateImage(cvGetSize(dst), 8, 3);
		cvCvtColor(dst, imageHSV, CV_BGR2HSV);

		ballColor = cvCreateImage(cvGetSize(src), IPL_DEPTH_8U, 1);
		//whiteRobotColor = cvCreateImage(cvGetSize(src), IPL_DEPTH_8U, 1);
		blueRobotColor = cvCreateImage(cvGetSize(src), IPL_DEPTH_8U, 1);

		cvClearMemStorage(storage);

		/*
		 * Filtering the blue robot color
		 */
		cvInRangeS(
				imageHSV,
				cvScalar(hEventHandler[1] - toleranceH[1], sEventHandler[1] - toleranceS[1], vEventHandler[1] - toleranceV[1]),
				cvScalar(hEventHandler[1] + toleranceH[1], sEventHandler[1] + toleranceS[1], vEventHandler[1] + toleranceV[1]),
				blueRobotColor
		);
		cvErode(blueRobotColor, blueRobotColor, NULL, 1);
		cvDilate(blueRobotColor, blueRobotColor, NULL, 1);

		/*
		 * Filtering the white robot color
		 */
//		cvInRangeS(
//				imageHSV,
//				cvScalar(hEventHandler[0] - toleranceH[0], sEventHandler[0] - toleranceS[0], vEventHandler[0] - toleranceV[0]),
//				cvScalar(hEventHandler[0] + toleranceH[0], sEventHandler[0] + toleranceS[0], vEventHandler[0] + toleranceV[0]),
//				whiteRobotColor
//		);
//		cvErode(whiteRobotColor, whiteRobotColor, NULL, 1);
//		cvDilate(whiteRobotColor, whiteRobotColor, NULL, 1);

		/*
		 * Filtering ball color
		 */
		cvInRangeS(
				imageHSV,
				cvScalar(hEventHandler[2] - toleranceH[2], sEventHandler[2] - toleranceS[2], vEventHandler[2] - toleranceV[2]),
				cvScalar(hEventHandler[2] + toleranceH[2], sEventHandler[2] + toleranceS[2], vEventHandler[2] + toleranceV[2]),
				ballColor
		);
		cvErode(ballColor, ballColor, NULL, 1);
		cvDilate(ballColor, ballColor, NULL, 1);

		getObjectPosition(&ballPosition, ballColor, 0, 0, ballColor->width - 1, ballColor->height - 1);
		getObjectPosition(&robotZeroBluePosition, blueRobotColor, 0, (ballColor->height / 2) - 1, (ballColor->width / 2) - 1 + FIELD_TOLERANCE , ballColor->height - 1);
		getObjectPosition(&robotOneBluePosition, blueRobotColor, (ballColor->width / 2) - 1 - FIELD_TOLERANCE, (ballColor->height / 2) - 1, ballColor->width - 1, ballColor->height - 1);
//		getObjectPosition(&robotZeroWhitePosition, whiteRobotColor, 0, (ballColor->height / 2) - 1, (ballColor->width / 2) - 1 + FIELD_TOLERANCE , ballColor->height - 1);
//		getObjectPosition(&robotOneWhitePosition, whiteRobotColor, (ballColor->width / 2) - 1 - FIELD_TOLERANCE, (ballColor->height / 2) - 1, ballColor->width - 1, ballColor->height - 1);

		//printf("BARICENTRE: %d, %d\n", robotZeroBluePosition.x, robotZeroBluePosition.y);
		//printMatrix(blueRobotColor, robotZeroBluePosition.x - 15, robotZeroBluePosition.y - 15, robotZeroBluePosition.x + 15, robotZeroBluePosition.y + 15, robotZeroBluePosition.x, robotZeroBluePosition.y);

		msgToSend.mtype = 1;
		msgToSend.xBall = ballPosition.x;
		msgToSend.yBall = ballPosition.y;
		//		robotZeroPosition.x = msgToSend.xLeftRobot = (int) (robotZeroBluePosition.x + robotZeroWhitePosition.x) / 2.0;
		//		robotZeroPosition.y = msgToSend.yLeftRobot = (int) (robotZeroBluePosition.y + robotZeroWhitePosition.y) / 2.0;
		//		robotOnePosition.x = msgToSend.xRightRobot = (int) (robotOneBluePosition.x + robotOneWhitePosition.x) / 2.0;
		//		robotOnePosition.y = msgToSend.yRightRobot = (int) (robotOneBluePosition.y + robotOneWhitePosition.y) / 2.0;

		robotZeroPosition.x = msgToSend.xLeftRobot = (int) robotZeroBluePosition.x;
		robotZeroPosition.y = msgToSend.yLeftRobot = (int) robotZeroBluePosition.y;
		robotOnePosition.x = msgToSend.xRightRobot = (int) robotOneBluePosition.x;
		robotOnePosition.y = msgToSend.yRightRobot = (int) robotOneBluePosition.y;

		if (
				ballPosition.x > 0 &&
				ballPosition.y > 0 &&
				msgToSend.xLeftRobot > 0 &&
				msgToSend.yLeftRobot > 0 &&
				msgToSend.xRightRobot > 0 &&
				msgToSend.yRightRobot > 0
		) {
			mq_send(msgQueueImageToIa, (char *) &msgToSend, sizeof(struct msgImageToIa), prio);
			ballPositionOld = ballPosition;

			cvCircle(dst, cvPoint(ballPosition.x, ballPosition.y), 15, cvScalar(0,0,255), 2);
		}

		cvLine(dst, cvPoint(0,HOME_LINE), cvPoint(480,HOME_LINE), cvScalar(0,255,0), 3);
		cvLine(dst, cvPoint(0,640 - HOME_LINE), cvPoint(480,640 - HOME_LINE), cvScalar(0,255,0), 3);
		cvLine(dst, cvPoint(240,0), cvPoint(240,640), cvScalar(0,0,0), 1);

		hScale = 0.4;
		vScale = 0.4;
		lineWidth = 1;

		cvInitFont(&font,CV_FONT_HERSHEY_SIMPLEX|CV_FONT_ITALIC, hScale,vScale,0,lineWidth);

		sprintf(cordRobot1, "%d, %d", robotZeroPosition.x, robotZeroPosition.y);
		sprintf(cordRobot2, "%d, %d", robotOnePosition.x, robotOnePosition.y);

		cvPutText (dst, cordRobot1,cvPoint(robotZeroPosition.x - 15,robotZeroPosition.y - 10), &font, cvScalar(0,0,0));
		cvPutText (dst, cordRobot2,cvPoint(robotOnePosition.x - 15,robotOnePosition.y- 10), &font, cvScalar(0,0,0));

		hScale = 0.8;
		vScale = 0.8;
		lineWidth = 2;

		cvInitFont(&font,CV_FONT_HERSHEY_SIMPLEX|CV_FONT_ITALIC, hScale,vScale,0,lineWidth);

		sprintf(alpha0String, "Alpha0: %.2f", alpha0);
		sprintf(alpha1String, "Alpha1: %.2f", alpha1);

		if (inAttackMode)
			sprintf(inAttackModeString, "In Attack");
		else
			sprintf(inAttackModeString, "In Defense");

		cvPutText (dst, alpha0String,cvPoint(10,20), &font, cvScalar(0,0,0));
		cvPutText (dst, alpha1String,cvPoint(10,50), &font, cvScalar(0,0,0));
		cvPutText (dst, inAttackModeString,cvPoint(10,80), &font, cvScalar(0,0,0));

		cvShowImage("PERSPECTIVE", dst);
		//cvShowImage("WHITE MASK", whiteRobotColor);
		cvShowImage("BLUE MASK", blueRobotColor);
		cvShowImage("BALL MASK", ballColor);

		cvReleaseImage(&src);
		cvReleaseImage(&dst);
		cvReleaseImage(&blueRobotColor);
		//cvReleaseImage(&whiteRobotColor);
		cvReleaseImage(&ballColor);
		cvReleaseImage(&imageHSV);
	}

	delete camera;

	return NULL;
}

void ImageProcessing::printMatrix(IplImage *image, int initialX, int initialY, int finalX, int finalY, int xB, int yB) {
	for (int j = initialX; j <= finalX; j++) {
		for (int i = initialY; i <= finalY; i++) {
			CvScalar color = cvGet2D(image, i, j);
			if(xB == j && yB == i)
				printf("M ");
			else if (color.val[0] == 255)
				printf("# ");
			else
				printf("- ");
		}
		printf("\n");
	}
}

void ImageProcessing::getObjectPosition(Position * position, IplImage *image,
		int initialX, int initialY, int finalX, int finalY) {

	unsigned int numberX = 0, numberY = 0;
	unsigned int sumX = 0, sumY = 0;
	for (int i = initialX; i <= finalX; i++) {
		for (int j = initialY; j <= finalY; j++) {
			CvScalar color = cvGet2D(image, j, i);
			if (color.val[0] == 255) {
				sumX += i;
				sumY += j;
				numberX++, numberY++;
			}
		}
	}

	if (numberX > 0) {
		float meanX, meanY;
		meanX = (float) sumX / numberX;
		meanY = (float) sumY / numberY;
		position->x = (int)meanX;
		position->y = (int)meanY;
	}
	else {
		position->x = -1;
		position->y = -1;
	}
}

void ImageProcessing::getRobotDirection(IplImage *image, int initialX, int initialY, int finalX, int finalY, Position * direction) {


}
//
//void getRobotDirectionAux() {
//	int i, j;
//	for (i = 0; ) {
//		for () {
//
//		}
//	}
//
//}


ImageProcessing::~ImageProcessing() {

}

