/*
 * KheperaController.cpp
 *
 *  Created on: Nov 19, 2011
 *      Author: henriquenogueira
 */

#include <errno.h>
#include <vector>
#include <string>
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <cstdarg>

#include "Global.h"
#include "SerialCommunication.h"
#include "KheperaController.h"

using namespace std;

KheperaController::KheperaController(string * device) {

	/*
	 * Establishing the serial communication
	 */
	this->serialCommunication = new SerialCommunication(device);

	/*
	 * Opening the connection
	 */
	this->serialCommunication->openConnection();
	configureSpeedProfil(40, 120, 40, 120);

}


bool KheperaController::isConnected() {

	/*
	 * Check the status of the serial communication link
	 */
	return this->serialCommunication->isOpened();
}

/**
 * Run at speed
 * speed - cm / s
 */
bool KheperaController::run(int speed, int time) {
	setMotorSpeed(speed, speed);

	usleep((int) 1 * (time - ( (int) (speed / 10) * MINIMUM_COMMANDE_INTERVAL )));
	for (int i = speed - 10; i >= 0; i -= 10) {
		this->setMotorSpeed(i, i);
		usleep(MINIMUM_COMMANDE_INTERVAL);
	}
	return true;
}

bool KheperaController::walkDistance(int distance, int speed)
{

	setPositionCounter(0,0);
	usleep(250000);
	setPositionToReach(125*distance, 125*distance);
	//TODO : Change the return statement of walkDistance
	return true;
}

bool KheperaController::turn(int alpha)
{

	if (alpha > 180)
		alpha = alpha - 360;

	int pulse = 1035*alpha/180;
	//setMotorSpeed(0,0);

	//TODO : voir si ne peut pas réduire davantage
	//usleep(MINIMUM_COMMANDE_INTERVAL);
	setPositionCounter(0,0);
	//usleep(MINIMUM_COMMANDE_INTERVAL);
	setPositionToReach(pulse*(-1), pulse);
	sleep(2);

	return true;
}

bool KheperaController::stop()
{
	setMotorSpeed(0,0);
	return true;
}

bool KheperaController::shoot(int robot)
{

	int pulse = 1020*2;
	//setMotorSpeed(0,0);
	configureSpeedProfil(50,120,50,120);
	usleep(300000);

	//TODO : voir si ne peut pas réduire davantage
	//usleep(MINIMUM_COMMANDE_INTERVAL);
	setPositionCounter(0,0);
	//usleep(MINIMUM_COMMANDE_INTERVAL);
	setPositionToReach(pulse*(-1) * robot, pulse * robot);
	sleep(2);
	configureSpeedProfil(40,120,40,120);
	inAttackMode = false;
	return true;

	//	setMotorSpeed(0,0);
	//	usleep(25000);
	//	setMotorSpeed(speed*(-1), speed);
	//	usleep(500000);
	//	setMotorSpeed(0,0);
	//configureSpeedProfil(speed, KHEPERA_ACCE, (-1)*speed, KHEPERA_ACCE);
	//turn(alpha);

	//return true;
}

bool KheperaController::configureSpeedController(int kp, int ki, int kd) {
	string commandType = kKHEPERA_CONFIGURE;

	/*
	 * Creating array with the string-formed arguments
	 */
	vector<string> argsVector =
			this->createVectorWithArguments(3, kp, ki, kd);

	/*
	 * Generating the command line
	 */
	string command = this->createCommandLine(commandType, &argsVector);

	/*
	 * Send the command request to the serial link and reading its response
	 */
	string response = *this->serialCommunication->sendMessage(&command);

	/*
	 * Verifying if the command has been received successfully.
	 */
	return this->validateResponse(&commandType, &response);
}

bool KheperaController::configureSpeedProfil(int maxSpeedLeft, int accLeft, int maxSpeedRight, int accRight)
{
	string commandType = kKHEPERA_CONF_SPEED_PROFILE;

	/*
	 * Creating array with the string-formed arguments
	 */
	vector<string> argsVector =
			this->createVectorWithArguments(4, maxSpeedLeft, accLeft, maxSpeedLeft, accRight);

	/*
	 * Generating the command line
	 */
	string command = this->createCommandLine(commandType, &argsVector);

	/*
	 * Send the command request to the serial link and reading its response
	 */
	string response = *this->serialCommunication->sendMessage(&command);

	/*
	 * Verifying if the command has been received successfully.
	 */

	//TODO : Change the return statement
	return true;
}

bool KheperaController::setPositionToReach(int leftPosition, int rightPosition) {

	string commandType = kKHEPERA_SET_POSITION_TO_REACH;

	/*
	 * Creating array with the string-formed arguments
	 */
	vector<string> argsVector =
			this->createVectorWithArguments(2, leftPosition, rightPosition);

	/*
	 * Generating the command line
	 */
	string command = this->createCommandLine(commandType, &argsVector);

	/*
	 * Send the command request to the serial link and reading its response
	 */
	string response = *this->serialCommunication->sendMessage(&command);

	/*
	 * Verifying if the command has been received successfully.
	 */
	return this->validateResponse(&commandType, &response);
}

bool KheperaController::setMotorSpeed(int leftMotorSpeed, int rightMotorSpeed) {

	string commandType = kKHEPERA_SET_SPEED;

	/*
	 * Creating array with the string-formed arguments
	 */
	vector<string> argsVector =
			this->createVectorWithArguments(2, leftMotorSpeed, rightMotorSpeed);

	/*
	 * Generating the command line
	 */
	string command = this->createCommandLine(commandType, &argsVector);

	/*
	 * Send the command request to the serial link and reading its response
	 */
	string response = *this->serialCommunication->sendMessage(&command);


	/*
	 * Verifying if the command has been received successfully.
	 */
	//return this->validateResponse(&commandType, &response);
	return true;
}

bool KheperaController::readPosition() {

	string commandType = kKHEPERA_READ_POSITION_COUNTER;

	/*
	 * Creating array with the string-formed arguments
	 */
	vector<string> argsVector =
			this->createVectorWithArguments(0);

	/*
	 * Generating the command line
	 */
	string command = this->createCommandLine(commandType, &argsVector);

	/*
	 * Send the command request to the serial link and reading its response
	 */
	string response = *this->serialCommunication->sendMessage(&command);


	/*
	 * Verifying if the command has been received successfully.
	 */
	//return this->validateResponse(&commandType, &response);
	return true;
}

bool KheperaController::setPositionCounter(int leftPosition, int rightPosition) {

	string commandType = kKHEPERA_SET_POSITION_COUNTER;

	/*
	 * Creating array with the string-formed arguments
	 */
	vector<string> argsVector =
			this->createVectorWithArguments(2, leftPosition, rightPosition);

	/*
	 * Generating the command line
	 */
	string command = this->createCommandLine(commandType, &argsVector);

	/*
	 * Send the command request to the serial link and reading its response
	 */
	string response = *this->serialCommunication->sendMessage(&command);

	/*
	 * Verifying if the command has been received successfully.
	 */
	return this->validateResponse(&commandType, &response);
}

vector<string> KheperaController::createVectorWithArguments(int argsNumber, ...) {

	/*
	 * Variable-sized list of all arguments passed to this function
	 */
	va_list argsList;

	/*
	 * Vector user to keep the arguments in a string type
	 */
	vector<string> args;

	/*
	 * Starting the arguments list
	 */
	va_start(argsList, argsNumber);

	for(int i = 0; i < argsNumber; i++) {

		/*
		 * Acquiring next argument
		 */
		int arg = va_arg(argsList, int);

		/*
		 * Creating a C string array of the argument retrieved
		 */
		char argCStr[MAX_PARAMETER_SIZE];
		sprintf(argCStr, "%d", arg);

		/*
		 * Pushing it to the arguments array
		 */
		string argStr(argCStr);
		args.push_back(argStr);
	}

	/*
	 * Freeing the arguments list passed to this method
	 */
	va_end(argsList);

	return args;
}

string KheperaController::createCommandLine(string command, vector<string> * args) {

	if (!this->isConnected()) {
		errno = ENXIO;
		perror("[KheperaController] Error on sending message");
	}

	/*
	 * Structure which keeps the command line
	 */
	string commandString = "";

	/*
	 * Appending the command code to the string command line
	 */
	commandString.append(command);

	/*
	 * Iterating the arguments list to cunstruct the arguments string array
	 */
	vector<string>::iterator it;
	for (it = args->begin(); it < args->end(); it++) {

		/*
		 * Arguments separator
		 */
		commandString.append(ARGUMENTS_SEPARATOR);
		commandString.append(*it);
	}

	return commandString;
}

bool KheperaController::validateResponse(string * command, string * response) {
	return true;

	char commandType = (*command)[0];
	char responseType = (*response)[0];

	if (commandType == toupper(responseType))
		return true;
	return false;
}

KheperaController::~KheperaController() {

	/*
	 * Killing serial communication link
	 */
	this->serialCommunication->closeConnection();
}

