/*
 * KheperaController.h
 *
 *  Created on: Nov 19, 2011
 *      Author: henriquenogueira
 */

#ifndef KHEPERACONTROLLER_H_
#define KHEPERACONTROLLER_H_

#include <string>
#include <vector>

/*
 * String size of each command argument
 */
#define MAX_PARAMETER_SIZE						20
#define ARGUMENTS_SEPARATOR 					","

/*
 * Khepera command codes
 */
#define KHEPERA_SPEED_SHOOT 					65
#define KHEPERA_SPEED_WALKING 					20
#define KHEPERA_ACCE							125

#define kKHEPERA_CONFIGURE 						"A"
#define kKHEPERA_READ_SOFTWARE_VERSION 			"B"
#define kKHEPERA_SET_POSITION_TO_REACH			"C"
#define kKHEPERA_SET_SPEED						"D"
#define kKHEPERA_READ_SPEED						"E"
#define kKHEPERA_CONF_POSITION_PID_CONTROLLER	"F"
#define kKHEPERA_SET_POSITION_COUNTER			"G"
#define kKHEPERA_READ_POSITION_COUNTER			"H"
#define kKHEPERA_READ_AD_INPUT					"I"
#define kKHEPERA_CONF_SPEED_PROFILE 			"J"
#define kKHEPERA_READ_STATUS_MOTION_CONTROLLER	"K"
#define KkHEPERA_CHANGE_LED_STATE 				"L"
#define kKHEPERA_READ_PROXIMITY_SENSORS 		"N"
#define kKHEPERA_READ_AMBIENT_LIGHT_SENSORS 	"O"
#define kKHEPERA_SET_PWM 						"P"
#define kKHEPERA_READ_BYTE_EXTENSION_BUS 		"R"
#define kKHEPERA_SEND_MESSAGE_TURRET 			"T"
#define kKHEPERA_WRITE_BYTE_EXTENSION_BUS 		"W"

class SerialCommunication;

class KheperaController {

public:

	KheperaController(std::string * device);

	/*
	 * Verifies if the serial connection is opened
	 */
	bool isConnected();

	/*
	 * Configure the Kp, Ki and Kd parameters of the speed controller
	 */

	/*
	 * Get the Khepera walk
	 * distance : distance to walk, in cm
	 */
	bool walkDistance(int distance, int speed = KHEPERA_SPEED_WALKING);

	/*
	 * Get the Khepera walk
	 * distance : distance to walk, in cm
	 */
	bool readPosition();

	/*
	 * Get the Khepera turning
	 */
	bool turn(int degree);

	/*
	 * Stop the Khepera
	 */
	bool stop();

	/*
	 *Get the khepera running
	 */

	bool shoot(int speed = KHEPERA_SPEED_SHOOT);

	bool run(int speed, int time);

	//TODO : Set these methods private
	bool configureSpeedController(int kp, int ki, int kd);

	bool configureSpeedProfil(int maxSpeedLeft, int accLeft, int maxSpeedRight, int accRight);

	/*
	 * Set position to be reached
	 */
	bool setPositionToReach(int leftPosition, int rightPosition);

	/*
	 * Set position to be reached
	 */
	bool setPositionCounter(int leftPosition, int rightPosition);

	/*
	 * Set the motor speed for both wheels
	 */
	bool setMotorSpeed(int leftMotorSpeed, int rightMotorSpeed);

	virtual ~KheperaController();

private:

	/*
	 * Link with the serial communication library
	 */
	SerialCommunication *serialCommunication;

	/*
	 * Internal method used to create the structure of the command line
	 * based on the command and the arguments given.
	 */
	std::string createCommandLine(std::string command, std::vector<std::string> * args);

	/*
	 * This method creates an array with the string forms of each argument
	 * and it's used to create the command structure
	 */
	std::vector<std::string> createVectorWithArguments(int argsNumber, ...);

	/*
	 * Method used to validate if a command has been successfully executed
	 * on the host device
	 */
	bool validateResponse(std::string * command, std::string * response);
};

#endif /* KHEPERACONTROLLER_H_ */
