/*
 * SerialCommunication.cpp
 *
 *  Created on: Nov 18, 2011
 *      Author: henriquenogueira
 */

#include "SerialCommunication.h"

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>

#include <iostream>
#include <string>

using namespace std;

SerialCommunication::SerialCommunication(string * device) {
	this->deviceName = device;
	this->connectionState = CLOSED;
	this->configuration = (struct termios *) malloc(sizeof(struct termios));
}

void SerialCommunication::openConnection() {

	/*
	 * Opening file descriptor for the serial port
	 */
	this->fileDescriptor = open(this->deviceName->c_str(), O_RDWR | O_NOCTTY | O_NONBLOCK);
	if (this->fileDescriptor < 0) {
		perror("[SerialCommunication] Error on opening device");
		exit(-1);
	}

	/*
	 * Configuring local modes
	 */
	this->configuration->c_lflag &= ~ECHO;

	/*
	 * Configuring input modes
	 */
	this->configuration->c_iflag  = IGNBRK;
	this->configuration->c_iflag |= IGNPAR;

	/*
	 * Configuring output modes
	 */
	this->configuration->c_oflag =  0;

	/*
	 * Configuring control modes
	 */
	this->configuration->c_cflag  =  CREAD;
	this->configuration->c_cflag |=  CLOCAL;
	this->configuration->c_cflag |=  B9600;
	this->configuration->c_cflag |=  CS8;
	this->configuration->c_cflag |=  CSTOPB;

	/*
	 * Configuring input and output speed rate
	 */
	if (cfsetispeed(this->configuration, BAUDRATE) < 0
			|| cfsetospeed(this->configuration, BAUDRATE) < 0) {
		perror("[SerialCommunication] Error on configuring device BAUD rate");
		exit(-1);
	}

	/*
	 * Cleaning input buffer and flushing attributes
	 */
	//tcflush(this->fileDescriptor, TCIOFLUSH);
	tcsetattr(this->fileDescriptor, TCSANOW, this->configuration);

	this->connectionState = OPENED;
}

bool SerialCommunication::isOpened() {
	if (this->connectionState == OPENED)
		return true;
	else
		return false;
}

std::string * SerialCommunication::sendMessage(string * command) {

	if (!this->isOpened()) {
		perror("[SerialCommunication] Error on sending message");
		exit(-1);
	}

	/*
	 * We need to clean up the I/O buffer before sending/receiving messages
	 */
	tcflush(this->fileDescriptor, TCIFLUSH);

	/*
	 * Treating the command string
	 */
	int commandStrSize = command->size() + 1;	// Remember '\0'!
	char * commandStr = (char *) malloc(commandStrSize + 1);
	sprintf(commandStr, MESSAGE_FORMAT, command->c_str(), MESSAGE_END);

	/*
	 * Sending the command request to the serial port
	 */
	write(this->fileDescriptor, commandStr, commandStrSize);


	//tcflush(this->fileDescriptor, TCIOFLUSH);
	/*
	 * Reading response from serial port
	 */
//	int responseSize = 0, lastByte = 0;
//	char responseStr[MAX_RESPONSE_LENGHT];
//	int readBytes = 1;
//	while (responseStr[lastByte] != '\n' && responseSize < MAX_RESPONSE_LENGHT && readBytes > 0) {
//		readBytes = read(this->fileDescriptor, responseStr + responseSize, 1);
//		responseSize += readBytes;
//		lastByte = responseSize - 1;
//	}

	/*
	 * Setting the NULL caracter at the end of the string
	 */
//	responseStr[responseSize - 2] = '\0';
//	string response(responseStr);

	/*
	 * Saving command
	 */
	//this->lastResponse = &response;


	//return null;
	//return this->lastResponse;
	return new std::string("OK");
}

string * SerialCommunication::getLastCommand() {
	return this->lastCommand;
}

void SerialCommunication::closeConnection() {
	if (!this->isOpened()) {
		perror("[SerialCommunication] Error on closing device");
		exit(-1);
	}

	this->connectionState = CLOSED;
	close(this->fileDescriptor);
}

SerialCommunication::~SerialCommunication() {
	if (this->connectionState == OPENED)
		this->closeConnection();
	this->deviceName = new string("");
	free((void *)this->configuration);
}

